@@ -15,6 +15,7 @@ pub fn builtin() -> Vec<App> {
         install::cli(),
         locate_project::cli(),
         login::cli(),
+        logout::cli(),
         metadata::cli(),
         new::cli(),
         owner::cli(),
@@ -52,6 +53,7 @@ pub fn builtin_exec(cmd: &str) -> Option<fn(&mut Config, &ArgMatches<'_>) -> Cli
         "install" => install::exec,
         "locate-project" => locate_project::exec,
         "login" => login::exec,
+        "logout" => logout::exec,
         "metadata" => metadata::exec,
         "new" => new::exec,
         "owner" => owner::exec,
@@ -90,6 +92,7 @@ pub mod init;
 pub mod install;
 pub mod locate_project;
 pub mod login;
+pub mod logout;
 pub mod metadata;
 pub mod new;
 pub mod owner;
