@@ -2,8 +2,8 @@
 //!
 //! See `cargo_test_support::cross_compile` for more detail.
 
-use cargo_test_support::rustc_host;
 use cargo_test_support::{basic_bin_manifest, basic_manifest, cross_compile, project};
+use cargo_test_support::{is_nightly, rustc_host};
 
 #[cargo_test]
 fn simple_cross() {
@@ -15,21 +15,21 @@ fn simple_cross() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r#"
-            fn main() {{
-                assert_eq!(std::env::var("TARGET").unwrap(), "{}");
-            }}
-        "#,
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
                 cross_compile::alternate()
             ),
         )
@@ -37,11 +37,11 @@ fn simple_cross() {
             "src/main.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -67,30 +67,30 @@ fn simple_cross_config() {
             ".cargo/config",
             &format!(
                 r#"
-            [build]
-            target = "{}"
-        "#,
+                    [build]
+                    target = "{}"
+                "#,
                 cross_compile::alternate()
             ),
         )
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r#"
-            fn main() {{
-                assert_eq!(std::env::var("TARGET").unwrap(), "{}");
-            }}
-        "#,
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
                 cross_compile::alternate()
             ),
         )
@@ -98,11 +98,11 @@ fn simple_cross_config() {
             "src/main.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -127,14 +127,14 @@ fn simple_deps() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            path = "../bar"
-        "#,
+                [dependencies.bar]
+                path = "../bar"
+            "#,
         )
         .file("src/main.rs", "extern crate bar; fn main() { bar::bar(); }")
         .build();
@@ -165,9 +165,9 @@ fn linker() {
             ".cargo/config",
             &format!(
                 r#"
-            [target.{}]
-            linker = "my-linker-tool"
-        "#,
+                    [target.{}]
+                    linker = "my-linker-tool"
+                "#,
                 target
             ),
         )
@@ -176,11 +176,11 @@ fn linker() {
             "src/foo.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -206,6 +206,96 @@ fn linker() {
         .run();
 }
 
+#[cargo_test]
+fn plugin_with_extra_dylib_dep() {
+    if cross_compile::disabled() {
+        return;
+    }
+    if !is_nightly() {
+        // plugins are unstable
+        return;
+    }
+
+    let foo = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+
+                [dependencies.bar]
+                path = "../bar"
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+                #![feature(plugin)]
+                #![plugin(bar)]
+
+                fn main() {}
+            "#,
+        )
+        .build();
+    let _bar = project()
+        .at("bar")
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "0.0.1"
+                authors = []
+
+                [lib]
+                name = "bar"
+                plugin = true
+
+                [dependencies.baz]
+                path = "../baz"
+            "#,
+        )
+        .file(
+            "src/lib.rs",
+            r#"
+                #![feature(plugin_registrar, rustc_private)]
+
+                extern crate baz;
+                extern crate rustc_driver;
+
+                use rustc_driver::plugin::Registry;
+
+                #[plugin_registrar]
+                pub fn foo(reg: &mut Registry) {
+                    println!("{}", baz::baz());
+                }
+            "#,
+        )
+        .build();
+    let _baz = project()
+        .at("baz")
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "baz"
+                version = "0.0.1"
+                authors = []
+
+                [lib]
+                name = "baz"
+                crate_type = ["dylib"]
+            "#,
+        )
+        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
+        .build();
+
+    let target = cross_compile::alternate();
+    foo.cargo("build --target").arg(&target).run();
+}
+
 #[cargo_test]
 fn cross_tests() {
     if !cross_compile::can_run_on_host() {
@@ -216,27 +306,27 @@ fn cross_tests() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            authors = []
-            version = "0.0.0"
+                [project]
+                name = "foo"
+                authors = []
+                version = "0.0.0"
 
-            [[bin]]
-            name = "bar"
-        "#,
+                [[bin]]
+                name = "bar"
+            "#,
         )
         .file(
             "src/bin/bar.rs",
             &format!(
                 r#"
-            #[allow(unused_extern_crates)]
-            extern crate foo;
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-            #[test] fn test() {{ main() }}
-        "#,
+                    #[allow(unused_extern_crates)]
+                    extern crate foo;
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                    #[test] fn test() {{ main() }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -244,10 +334,10 @@ fn cross_tests() {
             "src/lib.rs",
             &format!(
                 r#"
-            use std::env;
-            pub fn foo() {{ assert_eq!(env::consts::ARCH, "{}"); }}
-            #[test] fn test_foo() {{ foo() }}
-        "#,
+                    use std::env;
+                    pub fn foo() {{ assert_eq!(env::consts::ARCH, "{}"); }}
+                    #[test] fn test_foo() {{ foo() }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -279,11 +369,11 @@ fn no_cross_doctests() {
         .file(
             "src/lib.rs",
             r#"
-            //! ```
-            //! extern crate foo;
-            //! assert!(true);
-            //! ```
-        "#,
+                //! ```
+                //! extern crate foo;
+                //! assert!(true);
+                //! ```
+            "#,
         )
         .build();
 
@@ -358,11 +448,11 @@ fn simple_cargo_run() {
             "src/main.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -383,36 +473,36 @@ fn cross_with_a_build_script() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = 'build.rs'
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = 'build.rs'
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r#"
-            use std::env;
-            use std::path::PathBuf;
-            fn main() {{
-                assert_eq!(env::var("TARGET").unwrap(), "{0}");
-                let mut path = PathBuf::from(env::var_os("OUT_DIR").unwrap());
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "out");
-                path.pop();
-                assert!(path.file_name().unwrap().to_str().unwrap()
-                            .starts_with("foo-"));
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "build");
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "debug");
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "{0}");
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "target");
-            }}
-        "#,
+                    use std::env;
+                    use std::path::PathBuf;
+                    fn main() {{
+                        assert_eq!(env::var("TARGET").unwrap(), "{0}");
+                        let mut path = PathBuf::from(env::var_os("OUT_DIR").unwrap());
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "out");
+                        path.pop();
+                        assert!(path.file_name().unwrap().to_str().unwrap()
+                                    .starts_with("foo-"));
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "build");
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "debug");
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "{0}");
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "target");
+                    }}
+                "#,
                 target
             ),
         )
@@ -446,25 +536,25 @@ fn build_script_needed_for_host_and_target() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = 'build.rs'
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = 'build.rs'
 
-            [dependencies.d1]
-            path = "d1"
-            [build-dependencies.d2]
-            path = "d2"
-        "#,
+                [dependencies.d1]
+                path = "d1"
+                [build-dependencies.d2]
+                path = "d2"
+            "#,
         )
         .file(
             "build.rs",
             r#"
-            #[allow(unused_extern_crates)]
-            extern crate d2;
-            fn main() { d2::d2(); }
-        "#,
+                #[allow(unused_extern_crates)]
+                extern crate d2;
+                fn main() { d2::d2(); }
+            "#,
         )
         .file(
             "src/main.rs",
@@ -477,35 +567,35 @@ fn build_script_needed_for_host_and_target() {
         .file(
             "d1/Cargo.toml",
             r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
-            build = 'build.rs'
-        "#,
+                [package]
+                name = "d1"
+                version = "0.0.0"
+                authors = []
+                build = 'build.rs'
+            "#,
         )
         .file("d1/src/lib.rs", "pub fn d1() {}")
         .file(
             "d1/build.rs",
             r#"
-            use std::env;
-            fn main() {
-                let target = env::var("TARGET").unwrap();
-                println!("cargo:rustc-flags=-L /path/to/{}", target);
-            }
-        "#,
+                use std::env;
+                fn main() {
+                    let target = env::var("TARGET").unwrap();
+                    println!("cargo:rustc-flags=-L /path/to/{}", target);
+                }
+            "#,
         )
         .file(
             "d2/Cargo.toml",
             r#"
-            [package]
-            name = "d2"
-            version = "0.0.0"
-            authors = []
+                [package]
+                name = "d2"
+                version = "0.0.0"
+                authors = []
 
-            [dependencies.d1]
-            path = "../d1"
-        "#,
+                [dependencies.d1]
+                path = "../d1"
+            "#,
         )
         .file(
             "d2/src/lib.rs",
@@ -554,14 +644,14 @@ fn build_deps_for_the_right_arch() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
 
-            [dependencies.d2]
-            path = "d2"
-        "#,
+                [dependencies.d2]
+                path = "d2"
+            "#,
         )
         .file("src/main.rs", "extern crate d2; fn main() {}")
         .file("d1/Cargo.toml", &basic_manifest("d1", "0.0.0"))
@@ -569,15 +659,15 @@ fn build_deps_for_the_right_arch() {
         .file(
             "d2/Cargo.toml",
             r#"
-            [package]
-            name = "d2"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "d2"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
 
-            [build-dependencies.d1]
-            path = "../d1"
-        "#,
+                [build-dependencies.d1]
+                path = "../d1"
+            "#,
         )
         .file("d2/build.rs", "extern crate d1; fn main() {}")
         .file("d2/src/lib.rs", "")
@@ -597,40 +687,40 @@ fn build_script_only_host() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
 
-            [build-dependencies.d1]
-            path = "d1"
-        "#,
+                [build-dependencies.d1]
+                path = "d1"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .file("build.rs", "extern crate d1; fn main() {}")
         .file(
             "d1/Cargo.toml",
             r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
-        "#,
+                [package]
+                name = "d1"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file("d1/src/lib.rs", "pub fn d1() {}")
         .file(
             "d1/build.rs",
             r#"
-            use std::env;
+                use std::env;
 
-            fn main() {
-                assert!(env::var("OUT_DIR").unwrap().replace("\\", "/")
-                                           .contains("target/debug/build/d1-"),
-                        "bad: {:?}", env::var("OUT_DIR"));
-            }
-        "#,
+                fn main() {
+                    assert!(env::var("OUT_DIR").unwrap().replace("\\", "/")
+                                               .contains("target/debug/build/d1-"),
+                            "bad: {:?}", env::var("OUT_DIR"));
+                }
+            "#,
         )
         .build();
 
@@ -647,16 +737,16 @@ fn plugin_build_script_right_arch() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
 
-            [lib]
-            name = "foo"
-            plugin = true
-        "#,
+                [lib]
+                name = "foo"
+                plugin = true
+            "#,
         )
         .file("build.rs", "fn main() {}")
         .file("src/lib.rs", "")
@@ -688,15 +778,15 @@ fn build_script_with_platform_specific_dependencies() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
 
-            [build-dependencies.d1]
-            path = "d1"
-        "#,
+                [build-dependencies.d1]
+                path = "d1"
+            "#,
         )
         .file(
             "build.rs",
@@ -711,14 +801,14 @@ fn build_script_with_platform_specific_dependencies() {
             "d1/Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
+                    [package]
+                    name = "d1"
+                    version = "0.0.0"
+                    authors = []
 
-            [target.{}.dependencies]
-            d2 = {{ path = "../d2" }}
-        "#,
+                    [target.{}.dependencies]
+                    d2 = {{ path = "../d2" }}
+                "#,
                 host
             ),
         )
@@ -761,18 +851,18 @@ fn platform_specific_dependencies_do_not_leak() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
 
-            [dependencies.d1]
-            path = "d1"
+                [dependencies.d1]
+                path = "d1"
 
-            [build-dependencies.d1]
-            path = "d1"
-        "#,
+                [build-dependencies.d1]
+                path = "d1"
+            "#,
         )
         .file("build.rs", "extern crate d1; fn main() {}")
         .file("src/lib.rs", "")
@@ -780,14 +870,14 @@ fn platform_specific_dependencies_do_not_leak() {
             "d1/Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
+                    [package]
+                    name = "d1"
+                    version = "0.0.0"
+                    authors = []
 
-            [target.{}.dependencies]
-            d2 = {{ path = "../d2" }}
-        "#,
+                    [target.{}.dependencies]
+                    d2 = {{ path = "../d2" }}
+                "#,
                 host
             ),
         )
@@ -816,18 +906,18 @@ fn platform_specific_variables_reflected_in_build_scripts() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
+                    build = "build.rs"
 
-            [target.{host}.dependencies]
-            d1 = {{ path = "d1" }}
+                    [target.{host}.dependencies]
+                    d1 = {{ path = "d1" }}
 
-            [target.{target}.dependencies]
-            d2 = {{ path = "d2" }}
-        "#,
+                    [target.{target}.dependencies]
+                    d2 = {{ path = "d2" }}
+                "#,
                 host = host,
                 target = target
             ),
@@ -836,22 +926,22 @@ fn platform_specific_variables_reflected_in_build_scripts() {
             "build.rs",
             &format!(
                 r#"
-            use std::env;
+                    use std::env;
 
-            fn main() {{
-                let platform = env::var("TARGET").unwrap();
-                let (expected, not_expected) = match &platform[..] {{
-                    "{host}" => ("DEP_D1_VAL", "DEP_D2_VAL"),
-                    "{target}" => ("DEP_D2_VAL", "DEP_D1_VAL"),
-                    _ => panic!("unknown platform")
-                }};
+                    fn main() {{
+                        let platform = env::var("TARGET").unwrap();
+                        let (expected, not_expected) = match &platform[..] {{
+                            "{host}" => ("DEP_D1_VAL", "DEP_D2_VAL"),
+                            "{target}" => ("DEP_D2_VAL", "DEP_D1_VAL"),
+                            _ => panic!("unknown platform")
+                        }};
 
-                env::var(expected).ok()
-                    .expect(&format!("missing {{}}", expected));
-                env::var(not_expected).err()
-                    .expect(&format!("found {{}}", not_expected));
-            }}
-        "#,
+                        env::var(expected).ok()
+                            .expect(&format!("missing {{}}", expected));
+                        env::var(not_expected).err()
+                            .expect(&format!("found {{}}", not_expected));
+                    }}
+                "#,
                 host = host,
                 target = target
             ),
@@ -860,26 +950,26 @@ fn platform_specific_variables_reflected_in_build_scripts() {
         .file(
             "d1/Cargo.toml",
             r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
-            links = "d1"
-            build = "build.rs"
-        "#,
+                [package]
+                name = "d1"
+                version = "0.0.0"
+                authors = []
+                links = "d1"
+                build = "build.rs"
+            "#,
         )
         .file("d1/build.rs", r#"fn main() { println!("cargo:val=1") }"#)
         .file("d1/src/lib.rs", "")
         .file(
             "d2/Cargo.toml",
             r#"
-            [package]
-            name = "d2"
-            version = "0.0.0"
-            authors = []
-            links = "d2"
-            build = "build.rs"
-        "#,
+                [package]
+                name = "d2"
+                version = "0.0.0"
+                authors = []
+                links = "d2"
+                build = "build.rs"
+            "#,
         )
         .file("d2/build.rs", r#"fn main() { println!("cargo:val=1") }"#)
         .file("d2/src/lib.rs", "")
@@ -903,61 +993,61 @@ fn cross_test_dylib() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "foo"
-            crate_type = ["dylib"]
+                [lib]
+                name = "foo"
+                crate_type = ["dylib"]
 
-            [dependencies.bar]
-            path = "bar"
-        "#,
+                [dependencies.bar]
+                path = "bar"
+            "#,
         )
         .file(
             "src/lib.rs",
             r#"
-            extern crate bar as the_bar;
+                extern crate bar as the_bar;
 
-            pub fn bar() { the_bar::baz(); }
+                pub fn bar() { the_bar::baz(); }
 
-            #[test]
-            fn foo() { bar(); }
-        "#,
+                #[test]
+                fn foo() { bar(); }
+            "#,
         )
         .file(
             "tests/test.rs",
             r#"
-            extern crate foo as the_foo;
+                extern crate foo as the_foo;
 
-            #[test]
-            fn foo() { the_foo::bar(); }
-        "#,
+                #[test]
+                fn foo() { the_foo::bar(); }
+            "#,
         )
         .file(
             "bar/Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "bar"
-            crate_type = ["dylib"]
-        "#,
+                [lib]
+                name = "bar"
+                crate_type = ["dylib"]
+            "#,
         )
         .file(
             "bar/src/lib.rs",
             &format!(
                 r#"
-             use std::env;
-             pub fn baz() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                     use std::env;
+                     pub fn baz() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -977,3 +1067,52 @@ fn cross_test_dylib() {
         .with_stdout_contains_n("test foo ... ok", 2)
         .run();
 }
+
+#[cargo_test]
+fn doctest_xcompile_linker() {
+    if cross_compile::disabled() {
+        return;
+    }
+    if !is_nightly() {
+        // -Zdoctest-xcompile is unstable
+        return;
+    }
+
+    let target = cross_compile::alternate();
+    let p = project()
+        .file(
+            ".cargo/config",
+            &format!(
+                r#"
+                    [target.{}]
+                    linker = "my-linker-tool"
+                "#,
+                target
+            ),
+        )
+        .file("Cargo.toml", &basic_manifest("foo", "0.1.0"))
+        .file(
+            "src/lib.rs",
+            r#"
+                /// ```
+                /// assert_eq!(1, 1);
+                /// ```
+                pub fn foo() {}
+            "#,
+        )
+        .build();
+
+    // Fails because `my-linker-tool` doesn't actually exist.
+    p.cargo("test --doc -v -Zdoctest-xcompile --target")
+        .arg(&target)
+        .with_status(101)
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains(&format!(
+            "\
+[RUNNING] `rustdoc --crate-type lib --test [..]\
+    --target {target} [..] -C linker=my-linker-tool[..]
+",
+            target = target,
+        ))
+        .run();
+}
