@@ -1034,7 +1034,7 @@ fn new_warns_you_this_will_not_work() {
         .env("USER", "foo")
         .with_stderr(
             "\
-warning: compiling this new crate may not work due to invalid workspace configuration
+warning: compiling this new package may not work due to invalid workspace configuration
 
 current package believes it's in a workspace when it's not:
 current: [..]
@@ -1056,7 +1056,7 @@ fn new_warning_with_corrupt_ws() {
         .env("USER", "foo")
         .with_stderr(
             "\
-[WARNING] compiling this new crate may not work due to invalid workspace configuration
+[WARNING] compiling this new package may not work due to invalid workspace configuration
 
 failed to parse manifest at `[..]foo/Cargo.toml`
 
@@ -2319,3 +2319,80 @@ Caused by:
         )
         .run();
 }
+
+#[cargo_test]
+fn simple_primary_package_env_var() {
+    let is_primary_package = r#"
+        #[test]
+        fn verify_primary_package() {{
+            assert!(option_env!("CARGO_PRIMARY_PACKAGE").is_some());
+        }}
+    "#;
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [project]
+                name = "foo"
+                version = "0.1.0"
+                authors = []
+
+                [workspace]
+                members = ["bar"]
+            "#,
+        )
+        .file("src/lib.rs", is_primary_package)
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [project]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
+                workspace = ".."
+            "#,
+        )
+        .file("bar/src/lib.rs", is_primary_package);
+    let p = p.build();
+
+    p.cargo("test").run();
+
+    // Again, this time selecting a specific crate
+    p.cargo("clean").run();
+    p.cargo("test -p bar").run();
+
+    // Again, this time selecting all crates
+    p.cargo("clean").run();
+    p.cargo("test --all").run();
+}
+
+#[cargo_test]
+fn virtual_primary_package_env_var() {
+    let is_primary_package = r#"
+        #[test]
+        fn verify_primary_package() {{
+            assert!(option_env!("CARGO_PRIMARY_PACKAGE").is_some());
+        }}
+    "#;
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [workspace]
+                members = ["foo", "bar"]
+            "#,
+        )
+        .file("foo/Cargo.toml", &basic_manifest("foo", "0.1.0"))
+        .file("foo/src/lib.rs", is_primary_package)
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("bar/src/lib.rs", is_primary_package);
+    let p = p.build();
+
+    p.cargo("test").run();
+
+    // Again, this time selecting a specific crate
+    p.cargo("clean").run();
+    p.cargo("test -p foo").run();
+}
