@@ -9,5 +9,7 @@
 #[warn(clippy::drop_bounds)]
 #[warn(clippy::temporary_cstring_as_ptr)]
 #[warn(clippy::panic_params)]
+#[warn(clippy::unknown_clippy_lints)]
+#[warn(clippy::find_map)]
 
 fn main() {}
