@@ -12,6 +12,8 @@ impl Dlsym {
     pub fn from_str(name: &str) -> InterpResult<'static, Option<Dlsym>> {
         Ok(match &*name {
             "__pthread_get_minstack" => None,
+            "getrandom" => None, // std falls back to syscall(SYS_getrandom, ...) when this is NULL.
+            "statx" => None, // std falls back to syscall(SYS_statx, ...) when this is NULL.
             _ => throw_unsup_format!("unsupported Linux dlsym: {}", name),
         })
     }
@@ -27,7 +29,7 @@ pub trait EvalContextExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
     ) -> InterpResult<'tcx> {
         let this = self.eval_context_mut();
         let (_dest, _ret) = ret.expect("we don't support any diverging dlsym");
-        assert!(this.tcx.sess.target.target.target_os == "linux");
+        assert!(this.tcx.sess.target.os == "linux");
 
         match dlsym {}
     }
