@@ -1,4 +1,7 @@
-use ide_db::{defs::Definition, search::ReferenceKind};
+use ide_db::{
+    defs::Definition,
+    search::{FileReference, ReferenceKind},
+};
 use syntax::{
     ast::{self, AstNode, AstToken},
     TextRange,
@@ -16,7 +19,7 @@ use crate::{
 //
 // ```
 // fn main() {
-//     let x<|> = 1 + 2;
+//     let x$0 = 1 + 2;
 //     x * 4;
 // }
 // ```
@@ -44,8 +47,8 @@ pub(crate) fn inline_local_variable(acc: &mut Assists, ctx: &AssistContext) -> O
 
     let def = ctx.sema.to_def(&bind_pat)?;
     let def = Definition::Local(def);
-    let refs = def.usages(&ctx.sema).all();
-    if refs.is_empty() {
+    let usages = def.usages(&ctx.sema).all();
+    if usages.is_empty() {
         mark::hit!(test_not_applicable_if_variable_unused);
         return None;
     };
@@ -63,48 +66,46 @@ pub(crate) fn inline_local_variable(acc: &mut Assists, ctx: &AssistContext) -> O
         let_stmt.syntax().text_range()
     };
 
-    let mut wrap_in_parens = vec![true; refs.len()];
+    let wrap_in_parens = usages
+        .references
+        .values()
+        .flatten()
+        .map(|&FileReference { range, .. }| {
+            let usage_node =
+                ctx.covering_node_for_range(range).ancestors().find_map(ast::PathExpr::cast)?;
+            let usage_parent_option = usage_node.syntax().parent().and_then(ast::Expr::cast);
+            let usage_parent = match usage_parent_option {
+                Some(u) => u,
+                None => return Ok(false),
+            };
 
-    for (i, desc) in refs.iter().enumerate() {
-        let usage_node = ctx
-            .covering_node_for_range(desc.file_range.range)
-            .ancestors()
-            .find_map(ast::PathExpr::cast)?;
-        let usage_parent_option = usage_node.syntax().parent().and_then(ast::Expr::cast);
-        let usage_parent = match usage_parent_option {
-            Some(u) => u,
-            None => {
-                wrap_in_parens[i] = false;
-                continue;
-            }
-        };
-
-        wrap_in_parens[i] = match (&initializer_expr, usage_parent) {
-            (ast::Expr::CallExpr(_), _)
-            | (ast::Expr::IndexExpr(_), _)
-            | (ast::Expr::MethodCallExpr(_), _)
-            | (ast::Expr::FieldExpr(_), _)
-            | (ast::Expr::TryExpr(_), _)
-            | (ast::Expr::RefExpr(_), _)
-            | (ast::Expr::Literal(_), _)
-            | (ast::Expr::TupleExpr(_), _)
-            | (ast::Expr::ArrayExpr(_), _)
-            | (ast::Expr::ParenExpr(_), _)
-            | (ast::Expr::PathExpr(_), _)
-            | (ast::Expr::BlockExpr(_), _)
-            | (ast::Expr::EffectExpr(_), _)
-            | (_, ast::Expr::CallExpr(_))
-            | (_, ast::Expr::TupleExpr(_))
-            | (_, ast::Expr::ArrayExpr(_))
-            | (_, ast::Expr::ParenExpr(_))
-            | (_, ast::Expr::ForExpr(_))
-            | (_, ast::Expr::WhileExpr(_))
-            | (_, ast::Expr::BreakExpr(_))
-            | (_, ast::Expr::ReturnExpr(_))
-            | (_, ast::Expr::MatchExpr(_)) => false,
-            _ => true,
-        };
-    }
+            Ok(!matches!(
+                (&initializer_expr, usage_parent),
+                (ast::Expr::CallExpr(_), _)
+                    | (ast::Expr::IndexExpr(_), _)
+                    | (ast::Expr::MethodCallExpr(_), _)
+                    | (ast::Expr::FieldExpr(_), _)
+                    | (ast::Expr::TryExpr(_), _)
+                    | (ast::Expr::RefExpr(_), _)
+                    | (ast::Expr::Literal(_), _)
+                    | (ast::Expr::TupleExpr(_), _)
+                    | (ast::Expr::ArrayExpr(_), _)
+                    | (ast::Expr::ParenExpr(_), _)
+                    | (ast::Expr::PathExpr(_), _)
+                    | (ast::Expr::BlockExpr(_), _)
+                    | (ast::Expr::EffectExpr(_), _)
+                    | (_, ast::Expr::CallExpr(_))
+                    | (_, ast::Expr::TupleExpr(_))
+                    | (_, ast::Expr::ArrayExpr(_))
+                    | (_, ast::Expr::ParenExpr(_))
+                    | (_, ast::Expr::ForExpr(_))
+                    | (_, ast::Expr::WhileExpr(_))
+                    | (_, ast::Expr::BreakExpr(_))
+                    | (_, ast::Expr::ReturnExpr(_))
+                    | (_, ast::Expr::MatchExpr(_))
+            ))
+        })
+        .collect::<Result<Vec<_>, _>>()?;
 
     let init_str = initializer_expr.syntax().text().to_string();
     let init_in_paren = format!("({})", &init_str);
@@ -116,15 +117,16 @@ pub(crate) fn inline_local_variable(acc: &mut Assists, ctx: &AssistContext) -> O
         target,
         move |builder| {
             builder.delete(delete_range);
-            for (desc, should_wrap) in refs.iter().zip(wrap_in_parens) {
+            for (reference, should_wrap) in usages.references.values().flatten().zip(wrap_in_parens)
+            {
                 let replacement =
                     if should_wrap { init_in_paren.clone() } else { init_str.clone() };
-                match desc.kind {
+                match reference.kind {
                     ReferenceKind::FieldShorthandForLocal => {
                         mark::hit!(inline_field_shorthand);
-                        builder.insert(desc.file_range.range.end(), format!(": {}", replacement))
+                        builder.insert(reference.range.end(), format!(": {}", replacement))
                     }
-                    _ => builder.replace(desc.file_range.range, replacement),
+                    _ => builder.replace(reference.range, replacement),
                 }
             }
         },
@@ -146,7 +148,7 @@ mod tests {
             r"
 fn bar(a: usize) {}
 fn foo() {
-    let a<|> = 1;
+    let a$0 = 1;
     a + 1;
     if a > 10 {
     }
@@ -180,7 +182,7 @@ fn foo() {
             r"
 fn bar(a: usize) {}
 fn foo() {
-    let a<|> = 1 + 1;
+    let a$0 = 1 + 1;
     a + 1;
     if a > 10 {
     }
@@ -214,7 +216,7 @@ fn foo() {
             r"
 fn bar(a: usize) {}
 fn foo() {
-    let a<|> = bar(1);
+    let a$0 = bar(1);
     a + 1;
     if a > 10 {
     }
@@ -248,7 +250,7 @@ fn foo() {
             r"
 fn bar(a: usize): usize { a }
 fn foo() {
-    let a<|> = bar(1) as u64;
+    let a$0 = bar(1) as u64;
     a + 1;
     if a > 10 {
     }
@@ -281,7 +283,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = { 10 + 1 };
+    let a$0 = { 10 + 1 };
     a + 1;
     if a > 10 {
     }
@@ -313,7 +315,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = ( 10 + 1 );
+    let a$0 = ( 10 + 1 );
     a + 1;
     if a > 10 {
     }
@@ -346,7 +348,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let mut a<|> = 1 + 1;
+    let mut a$0 = 1 + 1;
     a + 1;
 }",
         );
@@ -358,7 +360,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = bar(10 + 1);
+    let a$0 = bar(10 + 1);
     let b = a * 10;
     let c = a as usize;
 }",
@@ -377,7 +379,7 @@ fn foo() {
             r"
 fn foo() {
     let x = vec![1, 2, 3];
-    let a<|> = x[0];
+    let a$0 = x[0];
     let b = a * 10;
     let c = a as usize;
 }",
@@ -397,7 +399,7 @@ fn foo() {
             r"
 fn foo() {
     let bar = vec![1];
-    let a<|> = bar.len();
+    let a$0 = bar.len();
     let b = a * 10;
     let c = a as usize;
 }",
@@ -421,7 +423,7 @@ struct Bar {
 
 fn foo() {
     let bar = Bar { foo: 1 };
-    let a<|> = bar.foo;
+    let a$0 = bar.foo;
     let b = a * 10;
     let c = a as usize;
 }",
@@ -445,7 +447,7 @@ fn foo() {
             r"
 fn foo() -> Option<usize> {
     let bar = Some(1);
-    let a<|> = bar?;
+    let a$0 = bar?;
     let b = a * 10;
     let c = a as usize;
     None
@@ -467,7 +469,7 @@ fn foo() -> Option<usize> {
             r"
 fn foo() {
     let bar = 10;
-    let a<|> = &bar;
+    let a$0 = &bar;
     let b = a * 10;
 }",
             r"
@@ -484,7 +486,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = (10, 20);
+    let a$0 = (10, 20);
     let b = a[0];
 }",
             r"
@@ -500,7 +502,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = [1, 2, 3];
+    let a$0 = [1, 2, 3];
     let b = a.len();
 }",
             r"
@@ -516,7 +518,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = (10 + 20);
+    let a$0 = (10 + 20);
     let b = a * 10;
     let c = a as usize;
 }",
@@ -535,7 +537,7 @@ fn foo() {
             r"
 fn foo() {
     let d = 10;
-    let a<|> = d;
+    let a$0 = d;
     let b = a * 10;
     let c = a as usize;
 }",
@@ -554,7 +556,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = { 10 };
+    let a$0 = { 10 };
     let b = a * 10;
     let c = a as usize;
 }",
@@ -572,7 +574,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = 10 + 20;
+    let a$0 = 10 + 20;
     let b = a * 10;
     let c = (a, 20);
     let d = [a, 10];
@@ -594,7 +596,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = vec![10, 20];
+    let a$0 = vec![10, 20];
     for i in a {}
 }",
             r"
@@ -610,7 +612,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = 1 > 0;
+    let a$0 = 1 > 0;
     while a {}
 }",
             r"
@@ -626,7 +628,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = 1 + 1;
+    let a$0 = 1 + 1;
     loop {
         break a;
     }
@@ -646,7 +648,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = 1 > 0;
+    let a$0 = 1 > 0;
     return a;
 }",
             r"
@@ -662,7 +664,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn foo() {
-    let a<|> = 1 > 0;
+    let a$0 = 1 > 0;
     match a {}
 }",
             r"
@@ -680,7 +682,7 @@ fn foo() {
             r"
 struct S { foo: i32}
 fn main() {
-    let <|>foo = 92;
+    let $0foo = 92;
     S { foo }
 }
 ",
@@ -700,7 +702,7 @@ fn main() {
             inline_local_variable,
             r"
 fn foo() {
-    let <|>a = 0;
+    let $0a = 0;
 }
             ",
         )
@@ -713,7 +715,7 @@ fn foo() {
             inline_local_variable,
             r"
 fn main() {
-    let x = <|>1 + 2;
+    let x = $01 + 2;
     x * 4;
 }
 ",
