@@ -1,6 +1,6 @@
 use syntax::{
     ast::{self, AstNode},
-    SyntaxElement, TextRange, TextSize, T,
+    match_ast, SyntaxElement, TextRange, TextSize, T,
 };
 
 use crate::{AssistContext, AssistId, AssistKind, Assists};
@@ -11,7 +11,7 @@ use crate::{AssistContext, AssistId, AssistKind, Assists};
 //
 // ```
 // fn main() {
-//     <|>dbg!(92);
+//     $0dbg!(92);
 // }
 // ```
 // ->
@@ -49,12 +49,29 @@ fn adjusted_macro_contents(macro_call: &ast::MacroCall) -> Option<String> {
         macro_text_with_brackets.len() - TextSize::of(')'),
     ));
 
-    let is_leaf = macro_call.syntax().next_sibling().is_none();
-    Some(if !is_leaf && needs_parentheses_around_macro_contents(contents) {
-        format!("({})", macro_text_in_brackets)
-    } else {
-        macro_text_in_brackets.to_string()
-    })
+    Some(
+        if !is_leaf_or_control_flow_expr(macro_call)
+            && needs_parentheses_around_macro_contents(contents)
+        {
+            format!("({})", macro_text_in_brackets)
+        } else {
+            macro_text_in_brackets.to_string()
+        },
+    )
+}
+
+fn is_leaf_or_control_flow_expr(macro_call: &ast::MacroCall) -> bool {
+    macro_call.syntax().next_sibling().is_none()
+        || match macro_call.syntax().parent() {
+            Some(parent) => match_ast! {
+                match parent {
+                    ast::Condition(_it) => true,
+                    ast::MatchExpr(_it) => true,
+                    _ => false,
+                }
+            },
+            None => false,
+        }
 }
 
 /// Verifies that the given macro_call actually matches the given name
@@ -93,8 +110,9 @@ fn needs_parentheses_around_macro_contents(macro_contents: Vec<SyntaxElement>) -
     if macro_contents.len() < 2 {
         return false;
     }
+    let mut macro_contents = macro_contents.into_iter().peekable();
     let mut unpaired_brackets_in_contents = Vec::new();
-    for element in macro_contents {
+    while let Some(element) = macro_contents.next() {
         match element.kind() {
             T!['('] | T!['['] | T!['{'] => unpaired_brackets_in_contents.push(element),
             T![')'] => {
@@ -117,7 +135,16 @@ fn needs_parentheses_around_macro_contents(macro_contents: Vec<SyntaxElement>) -
             }
             symbol_kind => {
                 let symbol_not_in_bracket = unpaired_brackets_in_contents.is_empty();
-                if symbol_not_in_bracket && symbol_kind.is_punct() {
+                if symbol_not_in_bracket
+                    && symbol_kind != T![:] // paths
+                    && (symbol_kind != T![.] // field/method access
+                        || macro_contents // range expressions consist of two SyntaxKind::Dot in macro invocations
+                            .peek()
+                            .map(|element| element.kind() == T![.])
+                            .unwrap_or(false))
+                    && symbol_kind != T![?] // try operator
+                    && (symbol_kind.is_punct() || symbol_kind == T![as])
+                {
                     return true;
                 }
             }
@@ -134,19 +161,19 @@ mod tests {
 
     #[test]
     fn test_remove_dbg() {
-        check_assist(remove_dbg, "<|>dbg!(1 + 1)", "1 + 1");
+        check_assist(remove_dbg, "$0dbg!(1 + 1)", "1 + 1");
 
-        check_assist(remove_dbg, "dbg!<|>((1 + 1))", "(1 + 1)");
+        check_assist(remove_dbg, "dbg!$0((1 + 1))", "(1 + 1)");
 
-        check_assist(remove_dbg, "dbg!(1 <|>+ 1)", "1 + 1");
+        check_assist(remove_dbg, "dbg!(1 $0+ 1)", "1 + 1");
 
-        check_assist(remove_dbg, "let _ = <|>dbg!(1 + 1)", "let _ = 1 + 1");
+        check_assist(remove_dbg, "let _ = $0dbg!(1 + 1)", "let _ = 1 + 1");
 
         check_assist(
             remove_dbg,
             "
 fn foo(n: usize) {
-    if let Some(_) = dbg!(n.<|>checked_sub(4)) {
+    if let Some(_) = dbg!(n.$0checked_sub(4)) {
         // ...
     }
 }
@@ -159,19 +186,21 @@ fn foo(n: usize) {
 }
 ",
         );
+
+        check_assist(remove_dbg, "$0dbg!(Foo::foo_test()).bar()", "Foo::foo_test().bar()");
     }
 
     #[test]
     fn test_remove_dbg_with_brackets_and_braces() {
-        check_assist(remove_dbg, "dbg![<|>1 + 1]", "1 + 1");
-        check_assist(remove_dbg, "dbg!{<|>1 + 1}", "1 + 1");
+        check_assist(remove_dbg, "dbg![$01 + 1]", "1 + 1");
+        check_assist(remove_dbg, "dbg!{$01 + 1}", "1 + 1");
     }
 
     #[test]
     fn test_remove_dbg_not_applicable() {
-        check_assist_not_applicable(remove_dbg, "<|>vec![1, 2, 3]");
-        check_assist_not_applicable(remove_dbg, "<|>dbg(5, 6, 7)");
-        check_assist_not_applicable(remove_dbg, "<|>dbg!(5, 6, 7");
+        check_assist_not_applicable(remove_dbg, "$0vec![1, 2, 3]");
+        check_assist_not_applicable(remove_dbg, "$0dbg(5, 6, 7)");
+        check_assist_not_applicable(remove_dbg, "$0dbg!(5, 6, 7");
     }
 
     #[test]
@@ -180,7 +209,7 @@ fn foo(n: usize) {
             remove_dbg,
             "
 fn foo(n: usize) {
-    if let Some(_) = dbg!(n.<|>checked_sub(4)) {
+    if let Some(_) = dbg!(n.$0checked_sub(4)) {
         // ...
     }
 }
@@ -197,7 +226,7 @@ fn foo(n: usize) {
         // the ast::MacroCall to include the semicolon at the end
         check_assist(
             remove_dbg,
-            r#"let res = <|>dbg!(1 * 20); // needless comment"#,
+            r#"let res = $0dbg!(1 * 20); // needless comment"#,
             r#"let res = 1 * 20; // needless comment"#,
         );
     }
@@ -209,7 +238,7 @@ fn foo(n: usize) {
             "
 fn main() {
     let mut a = 1;
-    while dbg!<|>(a) < 10000 {
+    while dbg!$0(a) < 10000 {
         a += 1;
     }
 }
@@ -229,12 +258,31 @@ fn main() {
     fn test_remove_dbg_keep_expression() {
         check_assist(
             remove_dbg,
-            r#"let res = <|>dbg!(a + b).foo();"#,
+            r#"let res = $0dbg!(a + b).foo();"#,
             r#"let res = (a + b).foo();"#,
         );
 
-        check_assist(remove_dbg, r#"let res = <|>dbg!(2 + 2) * 5"#, r#"let res = (2 + 2) * 5"#);
-        check_assist(remove_dbg, r#"let res = <|>dbg![2 + 2] * 5"#, r#"let res = (2 + 2) * 5"#);
+        check_assist(remove_dbg, r#"let res = $0dbg!(2 + 2) * 5"#, r#"let res = (2 + 2) * 5"#);
+        check_assist(remove_dbg, r#"let res = $0dbg![2 + 2] * 5"#, r#"let res = (2 + 2) * 5"#);
+    }
+
+    #[test]
+    fn test_remove_dbg_method_chaining() {
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(foo().bar()).baz();"#,
+            r#"let res = foo().bar().baz();"#,
+        );
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(foo.bar()).baz();"#,
+            r#"let res = foo.bar().baz();"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_field_chaining() {
+        check_assist(remove_dbg, r#"let res = $0dbg!(foo.bar).baz;"#, r#"let res = foo.bar.baz;"#);
     }
 
     #[test]
@@ -247,7 +295,7 @@ fn square(x: u32) -> u32 {
 }
 
 fn main() {
-    let x = square(dbg<|>!(5 + 10));
+    let x = square(dbg$0!(5 + 10));
     println!("{}", x);
 }"#,
             "dbg!(5 + 10)",
@@ -261,7 +309,7 @@ fn square(x: u32) -> u32 {
 }
 
 fn main() {
-    let x = square(dbg<|>!(5 + 10));
+    let x = square(dbg$0!(5 + 10));
     println!("{}", x);
 }"#,
             r#"
@@ -272,6 +320,101 @@ fn square(x: u32) -> u32 {
 fn main() {
     let x = square(5 + 10);
     println!("{}", x);
+}"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_try_expr() {
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(result?).foo();"#,
+            r#"let res = result?.foo();"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_await_expr() {
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(fut.await).foo();"#,
+            r#"let res = fut.await.foo();"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_as_cast() {
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(3 as usize).foo();"#,
+            r#"let res = (3 as usize).foo();"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_index_expr() {
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(array[3]).foo();"#,
+            r#"let res = array[3].foo();"#,
+        );
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(tuple.3).foo();"#,
+            r#"let res = tuple.3.foo();"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_range_expr() {
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(foo..bar).foo();"#,
+            r#"let res = (foo..bar).foo();"#,
+        );
+        check_assist(
+            remove_dbg,
+            r#"let res = $0dbg!(foo..=bar).foo();"#,
+            r#"let res = (foo..=bar).foo();"#,
+        );
+    }
+
+    #[test]
+    fn test_remove_dbg_followed_by_block() {
+        check_assist(
+            remove_dbg,
+            r#"fn foo() {
+    if $0dbg!(x || y) {}
+}"#,
+            r#"fn foo() {
+    if x || y {}
+}"#,
+        );
+        check_assist(
+            remove_dbg,
+            r#"fn foo() {
+    while let foo = $0dbg!(&x) {}
+}"#,
+            r#"fn foo() {
+    while let foo = &x {}
+}"#,
+        );
+        check_assist(
+            remove_dbg,
+            r#"fn foo() {
+    if let foo = $0dbg!(&x) {}
+}"#,
+            r#"fn foo() {
+    if let foo = &x {}
+}"#,
+        );
+        check_assist(
+            remove_dbg,
+            r#"fn foo() {
+    match $0dbg!(&x) {}
+}"#,
+            r#"fn foo() {
+    match &x {}
 }"#,
         );
     }
