@@ -3,19 +3,61 @@
 use std::{env, path::PathBuf, str};
 
 use anyhow::{bail, format_err, Context, Result};
-
-use crate::not_bash::{pushd, run};
+use xshell::{cmd, pushd};
 
 // Latest stable, feel free to send a PR if this lags behind.
-const REQUIRED_RUST_VERSION: u32 = 46;
+const REQUIRED_RUST_VERSION: u32 = 47;
 
 pub struct InstallCmd {
     pub client: Option<ClientOpt>,
     pub server: Option<ServerOpt>,
 }
 
+#[derive(Clone, Copy)]
 pub enum ClientOpt {
     VsCode,
+    VsCodeExploration,
+    VsCodeInsiders,
+    VsCodium,
+    VsCodeOss,
+    Any,
+}
+
+impl ClientOpt {
+    pub const fn as_cmds(&self) -> &'static [&'static str] {
+        match self {
+            ClientOpt::VsCode => &["code"],
+            ClientOpt::VsCodeExploration => &["code-exploration"],
+            ClientOpt::VsCodeInsiders => &["code-insiders"],
+            ClientOpt::VsCodium => &["codium"],
+            ClientOpt::VsCodeOss => &["code-oss"],
+            ClientOpt::Any => &["code", "code-exploration", "code-insiders", "codium", "code-oss"],
+        }
+    }
+}
+
+impl Default for ClientOpt {
+    fn default() -> Self {
+        ClientOpt::Any
+    }
+}
+
+impl std::str::FromStr for ClientOpt {
+    type Err = anyhow::Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        [
+            ClientOpt::VsCode,
+            ClientOpt::VsCodeExploration,
+            ClientOpt::VsCodeInsiders,
+            ClientOpt::VsCodium,
+            ClientOpt::VsCodeOss,
+        ]
+        .iter()
+        .copied()
+        .find(|c| [s] == c.as_cmds())
+        .ok_or_else(|| anyhow::format_err!("no such client"))
+    }
 }
 
 pub struct ServerOpt {
@@ -25,6 +67,7 @@ pub struct ServerOpt {
 pub enum Malloc {
     System,
     Mimalloc,
+    Jemalloc,
 }
 
 impl InstallCmd {
@@ -75,38 +118,35 @@ fn fix_path_for_mac() -> Result<()> {
     Ok(())
 }
 
-fn install_client(ClientOpt::VsCode: ClientOpt) -> Result<()> {
+fn install_client(client_opt: ClientOpt) -> Result<()> {
     let _dir = pushd("./editors/code");
 
     let find_code = |f: fn(&str) -> bool| -> Result<&'static str> {
-        ["code", "code-insiders", "codium", "code-oss"]
-            .iter()
-            .copied()
-            .find(|bin| f(bin))
-            .ok_or_else(|| {
-                format_err!("Can't execute `code --version`. Perhaps it is not in $PATH?")
-            })
+        client_opt.as_cmds().iter().copied().find(|bin| f(bin)).ok_or_else(|| {
+            format_err!("Can't execute `code --version`. Perhaps it is not in $PATH?")
+        })
     };
 
     let installed_extensions = if cfg!(unix) {
-        run!("npm --version").context("`npm` is required to build the VS Code plugin")?;
-        run!("npm install")?;
+        cmd!("npm --version").run().context("`npm` is required to build the VS Code plugin")?;
+        cmd!("npm install").run()?;
 
-        run!("npm run package --scripts-prepend-node-path")?;
+        cmd!("npm run package --scripts-prepend-node-path").run()?;
 
-        let code = find_code(|bin| run!("{} --version", bin).is_ok())?;
-        run!("{} --install-extension rust-analyzer.vsix --force", code)?;
-        run!("{} --list-extensions", code; echo = false)?
+        let code = find_code(|bin| cmd!("{bin} --version").read().is_ok())?;
+        cmd!("{code} --install-extension rust-analyzer.vsix --force").run()?;
+        cmd!("{code} --list-extensions").read()?
     } else {
-        run!("cmd.exe /c npm --version")
+        cmd!("cmd.exe /c npm --version")
+            .run()
             .context("`npm` is required to build the VS Code plugin")?;
-        run!("cmd.exe /c npm install")?;
+        cmd!("cmd.exe /c npm install").run()?;
 
-        run!("cmd.exe /c npm run package")?;
+        cmd!("cmd.exe /c npm run package").run()?;
 
-        let code = find_code(|bin| run!("cmd.exe /c {}.cmd --version", bin).is_ok())?;
-        run!(r"cmd.exe /c {}.cmd --install-extension rust-analyzer.vsix --force", code)?;
-        run!("cmd.exe /c {}.cmd --list-extensions", code; echo = false)?
+        let code = find_code(|bin| cmd!("cmd.exe /c {bin}.cmd --version").read().is_ok())?;
+        cmd!("cmd.exe /c {code}.cmd --install-extension rust-analyzer.vsix --force").run()?;
+        cmd!("cmd.exe /c {code}.cmd --list-extensions").read()?
     };
 
     if !installed_extensions.contains("rust-analyzer") {
@@ -122,7 +162,7 @@ fn install_client(ClientOpt::VsCode: ClientOpt) -> Result<()> {
 
 fn install_server(opts: ServerOpt) -> Result<()> {
     let mut old_rust = false;
-    if let Ok(stdout) = run!("cargo --version") {
+    if let Ok(stdout) = cmd!("cargo --version").read() {
         if !check_version(&stdout, REQUIRED_RUST_VERSION) {
             old_rust = true;
         }
@@ -134,12 +174,14 @@ fn install_server(opts: ServerOpt) -> Result<()> {
             REQUIRED_RUST_VERSION,
         )
     }
-
-    let malloc_feature = match opts.malloc {
-        Malloc::System => "",
-        Malloc::Mimalloc => "--features mimalloc",
+    let features = match opts.malloc {
+        Malloc::System => &[][..],
+        Malloc::Mimalloc => &["--features", "mimalloc"],
+        Malloc::Jemalloc => &["--features", "jemalloc"],
     };
-    let res = run!("cargo install --path crates/rust-analyzer --locked --force {}", malloc_feature);
+
+    let cmd = cmd!("cargo install --path crates/rust-analyzer --locked --force --features force-always-assert {features...}");
+    let res = cmd.run();
 
     if res.is_err() && old_rust {
         eprintln!(
@@ -148,7 +190,8 @@ fn install_server(opts: ServerOpt) -> Result<()> {
         );
     }
 
-    res.map(drop)
+    res?;
+    Ok(())
 }
 
 fn check_version(version_output: &str, min_minor_version: u32) -> bool {
